# Dependency structure for all main risks
getDependencyRisks <- function(correlation){
  dependencyRisks <- getDependency(correlation = correlation, labelIndependentComponents = c("scenario", "constant"))
  
  retClass(dependencyRisks, "dependencyRisks")
}


# Dependency within credit risk
getDependencyCredit <- function(corr){
  correlation <- getCorrelation(corr = corr, labels = c("creditBasel", "creditMerton"))
  dependencyCredit <- getDependency(correlation)
  
  retClass(dependencyCredit, "dependencyCredit")
}


# Dependency within market risk
getDependencyMarket <- function(){
  correlation <- getCorrelation(corr = 1, labels = c("marketInstruments", "marketParticipation"))
  dependencyMarket <- getDependency(correlation)
  
  retClass(dependencyMarket, "dependencyMarket")
}


# Dependency structure
getDependency <- function(correlation, labelIndependentComponents = character(0)){
  # The dependency consists of a correlation matrix and additional components that are independent of all risk factors
  x <- list(correlation = correlation, labelIndependentComponents = labelIndependentComponents)
  retClass(x, "Reordering")
}


# Transform a correlation parameter into a 2x2 correlation matrix
getCorrelation <- function(corr, labels){
  correlation <- matrix(c(1, corr, corr, 1), nrow = 2, dimnames = list(labels, labels))
  return(correlation)
}


# Apply the reordering to the rows of the table and concatenate everything into one table
getSimulationReordering <- function(sim, ordering){
  riskTypes <- names(which(!sapply(sim, is.null)))
  reordering <- lapply(stats::setNames(nm = riskTypes), function(x){sim[[x]][ordering[[x]]]})
  sim <- do.call(cbind, reordering)
  return(sim)
}


# Get the row permutation from the ranking
getOrdering <- function(ranks, main){
  ordering <- as.data.table(lapply(stats::setNames(nm = names(main)), function(x){order(main[[x]])[ranks[[x]]]}))
  
  # Without loss of generality, we can sort the rows such that the market risk factors are not affected by the reordering
  if("market" %in% names(ordering)){
    setorder(ordering, "market")  
  }
  
  return(ordering)
}