internalUpdateWorkbook <- function(){

  # Standartized way to specify how a range from the SST-Template will be converted into a table
  setTable <- function(keyword, description, sheet, row, isValue, columns, isEmptyAllowed = FALSE, isDropEmptyRows = FALSE, isCorrelation = FALSE){
    colParam = rbindlist(columns)
    setorder(colParam, "col")
    colParam[, ':=' (keyword = keyword, description = description, sheet = sheet, row = row, isValue = isValue, isEmptyAllowed = isEmptyAllowed, isDropEmptyRows = isDropEmptyRows, isCorrelation = isCorrelation)]
    return(colParam[])
  }
  
  # Standartized way to specify column parameters
  setColumn <- function(name, col, type, isValueRequired = TRUE, isReplacementZero = FALSE, characteristic = NA, dataSelection = NA, warning = character(0), error = character(0), filterValue = NA){
    col <- col + seq_len(length(name)) - 1
    colParam <- data.table(name = as.character(name),
                           col = as.integer(col),
                           type = as.character(type),
                           isValueRequired = as.logical(isValueRequired),
                           isReplacementZero = as.logical(isReplacementZero),
                           characteristic = as.character(characteristic),
                           dataSelection = as.character(dataSelection),
                           filterValue = as.character(filterValue),
                           Warning = paste0(warning, collapse = ","),
                           Error = paste0(error, collapse = ",")
    )
    
    return(colParam)
  }
  
  # Configuration of all tables
  TemplateConfig <- list(
    
    setTable(keyword = "correlationMarket",
             description = "correlation matrix",
             sheet = "Market (Dynamic)",
             row = 7,
             isValue = FALSE,
             isEmptyAllowed = TRUE,
             isCorrelation = TRUE,
             columns = list(
               setColumn(name = "name", col = 6, type = "Text", characteristic = "Primary key"),
               setColumn(name = paste0("RF", 1:100), col = 7, type = "Numeric")
             )
    )
    
    ,setTable(keyword = "configurationFDS",
              description = "FDS configuration",
              sheet = "Configuration",
              row = 3, #Row 2 contains the version ID
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              
              columns = list(
                setColumn(name = "action", col = 20, type = "Text"),
                setColumn(name = "row1", col = 21, type = "Integer"),
                setColumn(name = "row2", col = 22, type = "Integer", isValueRequired = FALSE),
                setColumn(name = "col1", col = 23, type = "Integer"),
                setColumn(name = "col2", col = 24, type = "Integer", isValueRequired = FALSE),
                setColumn(name = "textValue", col = 25, type = "Text", isValueRequired = FALSE),
                setColumn(name = "numericValue", col = 26, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE),
                setColumn(name = "sheet", col = 27, type = "Text", isValueRequired = FALSE),
                setColumn(name = "withFX", col = 28, type = "Integer", isValueRequired = FALSE),
                setColumn(name = "style", col = 29, type = "Text", isValueRequired = FALSE),
                setColumn(name = "sheetSource", col = 30, type = "Text", isValueRequired = FALSE)
              )
    )
    
    ,setTable(keyword = "correlationHealth",
              description = "correlation matrix health risk",
              sheet = "Correlation",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              isCorrelation = TRUE,
              columns = list(
                setColumn(name = "name", col = 14, type = "Text", characteristic = "Primary key"),
                setColumn(name = "Individual health", col = 15, type = "Numeric"),
                setColumn(name = "Collective compensation", col = 16, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "correlationLife",
              description = "correlation matrix life risk",
              sheet = "Correlation",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              isCorrelation = TRUE,
              columns = list(
                setColumn(name = "name", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "Mortality", col = 4, type = "Numeric"),
                setColumn(name = "Longevity", col = 5, type = "Numeric"),
                setColumn(name = "Disability", col = 6, type = "Numeric"),
                setColumn(name = "Reactivation", col = 7, type = "Numeric"),
                setColumn(name = "Costs", col = 8, type = "Numeric"),
                setColumn(name = "Lapses", col = 9, type = "Numeric"),
                setColumn(name = "Capital option", col = 10, type = "Numeric"),
                setColumn(name = "Cost BVG", col = 11, type = "Numeric"),
                setColumn(name = "Lapses BVG", col = 12, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "correlationRiskAggregation",
              description = "correlation matrix aggregation risks",
              sheet = "Correlation",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              isCorrelation = TRUE,
              columns = list(
                setColumn(name = "name", col = 18, type = "Text", characteristic = "Primary key"),
                setColumn(name = "market", col = 19, type = "Numeric"),
                setColumn(name = "life", col = 20, type = "Numeric"),
                setColumn(name = "health", col = 22, type = "Numeric"),
                setColumn(name = "nonlife", col = 21, type = "Numeric"),
                setColumn(name = "credit", col = 23, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "expectedFinancialResultTable",
              description = "expected financial result table",
              sheet = "Expected Financial Result",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "type", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "return", col = 3, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE),
                setColumn(name = "exposure", col = 5, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, warning = "Negative")
              )
    )
    
    ,setTable(keyword = "maturitiesInterestRate",
              description = "projection times",
              sheet = "Market (Static)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "time", col = 15, type = "Integer", characteristic = "Primary key"),
                setColumn(name = "projection", col = 16, type = "Text", isValueRequired = FALSE, isReplacementZero = TRUE)
              )
    )
    
    ,setTable(keyword = "initialFX",
              description = "exchange rates table",
              sheet = "Market Initial Values",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "from", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "to", col = 3, type = "Text", characteristic = "Primary key"),
                setColumn(name = "fx", col = 4, type = "Numeric", error = c("Negative", "Zero"))
              )
    )
    
    ,setTable(keyword = "initialInterestRate",
              description = "zero coupon rates table",
              sheet = "Market Initial Values",
              row = 6,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "currency", col = 6, type = "Text", characteristic = "Primary key", error = "notCurrency"),
                setColumn(name = as.character(1:50), col = 7, type = "Numeric", characteristic = "Transpose")
              )
    )
    
    ,setTable(keyword = "initialInterestRate1",
              description = "zero coupon rates table",
              sheet = "Market Initial Values",
              row = 6,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "currency", col = 6, type = "Text", characteristic = "Primary key", error = "notCurrency"),
                setColumn(name = paste0("Y", 1:50), col = 7, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "instrumentAsset",
              description = "asset prices table",
              sheet = "Asset Prices",
              row = 8,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "label", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "currency", col = 4, type = "Text", characteristic = "Primary key", error = "notCurrency"),
                setColumn(name = "value", col = 6, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, warning = "Negative")
              )
    )
    
    ,setTable(keyword = "instrumentAssetForward",
              description = "asset prices forward contracts table",
              sheet = "Forwards",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "contract", col = 2, type = "Text", characteristic = "Primary key", dataSelection = "Skip"),
                setColumn(name = "type", col = 4, type = "Text"),
                setColumn(name = "currency", col = 5, type = "Text", error = "notCurrency"),
                setColumn(name = "time", col = 6, type = "Integer", error = c("Negative", "Zero", "Larger50")),
                setColumn(name = "exposure", col = 7, type = "Numeric", warning = c("Zero"), error = "Negative"),
                setColumn(name = "price", col = 8, type = "Numeric", error = "Negative"),
                setColumn(name = "position", col = 9, type = "Text", error = "notLongShort")
              )
    )
    
    ,setTable(keyword = "instrumentDelta",
              description = "delta sensitivities table",
              sheet = "Delta Terms",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "name", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "unit", col = 4, type = "Text", error = "notUnit"),
                setColumn(name = "exposure", col = 12, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "instrumentFixedIncome",
              description = "fixed income table",
              sheet = "Fixed Income",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "currency", col = 2, type = "Text", characteristic = "Primary key", error = "notCurrency"),
                setColumn(name = "rating", col = 3, type = "Text", characteristic = "Primary key"),
                setColumn(name = "marketValue", col = 5, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, warning = "Negative"),
                setColumn(name = "spread", col = 6, type = "Numeric", isValueRequired = FALSE),
                setColumn(name = as.character(1:50), col = 7, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, characteristic = "Transpose", warning = "Negative")
              )
    )
    
    ,setTable(keyword = "instrumentFX_Forward",
              description = "fx forward contracts table",
              sheet = "Forwards",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "contract", col = 11, type = "Text", characteristic = "Primary key", dataSelection = "Skip"),
                setColumn(name = "time", col = 13, type = "Integer", error = c("Negative", "Zero", "Larger50")),
                setColumn(name = "nominal", col = 14, type = "Numeric", error = "Negative"),
                setColumn(name = "rate", col = 15, type = "Numeric", error = "Negative"),
                setColumn(name = "foreign", col = 16, type = "Text"),
                setColumn(name = "position", col = 17, type = "Text", error = "notLongShort")
              )
    )
    
    ,setTable(keyword = "instrumentLiability",
              description = "insurance liability cashflows",
              sheet = "Insurance Cashflows",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "type", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "currency", col = 3, type = "Text", characteristic = "Primary key", error = "notCurrency"),
                setColumn(name = as.character(1:50), col = 5, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, characteristic = "Transpose", warning = "Negative")
              )
    )
    
    ,setTable(keyword = "marketRiskFactorsAdditionalCheck",
              description = "risk factors mapping table",
              sheet = "Market (Static)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "factorId", col = 2, type = "Text"),
                setColumn(name = "type", col = 3, type = "Text"),
                setColumn(name = "name", col = 10, type = "Text"),
                setColumn(name = "scale", col = 11, type = "Numeric"),
                setColumn(name = "scaling", col = 12, type = "Text"),
                setColumn(name = "standalones", col = 13, type = "Text", characteristic = "Split", error = "notStandalone")
              )
    )
    ,setTable(keyword = "marketRiskFactorsAsset",
              description = "'asset price' risk factors",
              sheet = "Market (Static)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "factorId", col = 2, type = "Text"),
                setColumn(name = "type", col = 3, type = "Text", filterValue = "^asset price$"),
                setColumn(name = "currency", col = 4, type = "Text", characteristic = "Primary key", error = "notCurrency"),
                setColumn(name = "label", col = 5, type = "Text", characteristic = "Primary key"),
                setColumn(name = "targetCurrency", col = 6, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "horizon", col = 7, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "rating", col = 8, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "description", col = 9, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "name", col = 10, type = "Text"),
                setColumn(name = "scale", col = 11, type = "Numeric"),
                setColumn(name = "scaling", col = 12, type = "Text")
              )
    )
    
    ,setTable(keyword = "marketRiskFactorsCurrency",
              description = "'currency' risk factors",
              sheet = "Market (Static)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "factorId", col = 2, type = "Text"),
                setColumn(name = "type", col = 3, type = "Text", filterValue = "^currency$"),
                setColumn(name = "currency", col = 4, type = "Text", characteristic = "Primary key", error = c("notCurrency")),
                setColumn(name = "label", col = 5, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "targetCurrency", col = 6, type = "Text", error = c("notCurrency")),
                setColumn(name = "horizon", col = 7, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "rating", col = 8, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "description", col = 9, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "name", col = 10, type = "Text"),
                setColumn(name = "scale", col = 11, type = "Numeric"),
                setColumn(name = "scaling", col = 12, type = "Text")
              )
    )
    
    ,setTable(keyword = "marketRiskFactorsRate",
              description = "'rate' risk factors",
              sheet = "Market (Static)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "factorId", col = 2, type = "Text"),
                setColumn(name = "type", col = 3, type = "Text", filterValue = "^rate$"),
                setColumn(name = "currency", col = 4, type = "Text", characteristic = "Primary key", error = c("notCurrency")),
                setColumn(name = "label", col = 5, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "targetCurrency", col = 6, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "horizon", col = 7, type = "Text", isValueRequired = FALSE, characteristic = "Primary key", error = c("notHorizon")),
                setColumn(name = "rating", col = 8, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "description", col = 9, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "name", col = 10, type = "Text"),
                setColumn(name = "scale", col = 11, type = "Numeric"),
                setColumn(name = "scaling", col = 12, type = "Text")
              )
    )
    
    ,setTable(keyword = "marketRiskFactorsSpread",
              description = "'spread' risk factors",
              sheet = "Market (Static)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "factorId", col = 2, type = "Text"),
                setColumn(name = "type", col = 3, type = "Text", filterValue = "^spread$"),
                setColumn(name = "currency", col = 4, type = "Text", characteristic = "Primary key", error = c("notCurrency")),
                setColumn(name = "label", col = 5, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "targetCurrency", col = 6, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "horizon", col = 7, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "rating", col = 8, type = "Text", characteristic = "Primary key"),
                setColumn(name = "description", col = 9, type = "Text", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "name", col = 10, type = "Text"),
                setColumn(name = "scale", col = 11, type = "Numeric"),
                setColumn(name = "scaling", col = 12, type = "Text")
              )
    )
    
    ,setTable(keyword = "marketRiskFactorsDelta",
              description = "original risk factors (i.e. the risk factors that are included in the correlation matrix)",
              sheet = "Market (Static)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "factorId", col = 2, type = "Text"),
                setColumn(name = "name", col = 10, type = "Text", characteristic = "Primary key"),
                setColumn(name = "scale", col = 11, type = "Numeric"),
                setColumn(name = "scaling", col = 12, type = "Text", filterValue = "^Yes$")
              )
    )
    
    ,setTable(keyword = "marketRiskFactorsStandalones",
              description = "standalone definition",
              sheet = "Market (Static)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "factorId", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "standalones", col = 13, type = "Text", characteristic = "Split", error = c("notStandalone"))
              )
    )
    
    ,setTable(keyword = "marketVolatility",
              description = "volatilty table",
              sheet = "Market (Dynamic)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "name", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "volatility", col = 4, type = "Numeric", characteristic = "Named vector", error = c("Zero", "Negative"))
              )
    )
    
    ,setTable(keyword = "MVM_CashFlowsLife",
              description = "MVM life cashflows",
              sheet = "Life (MVM)",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "name", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = as.character(0:49), col = 6, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, characteristic = "Transpose")
              )
    )
    
    ,setTable(keyword = "nonlifeCDF",
              description = "cdf for nonlife risk",
              sheet = "Non Life",
              row = 18,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "x", col = 4, type = "Numeric", error = c("notStrictlyIncreasing")),
                setColumn(name = "cdf", col = 5, type = "Numeric", error = c("notContainOne", "notIncreasing", "Negative"))
              )
    )
    
    ,setTable(keyword = "nonlifeSimulation",
              description = "simulation vector for nonlife risk",
              sheet = "Non Life",
              row = 18,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "simulation", col = 2, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "scenarioMacroEconomic",
              description = "macroeconomic scenario table",
              sheet = "Macroeconomic Scenarios",
              row = 6,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "name", col = 2, type = "Text", characteristic = "Primary key", error = c("notRiskFactor")),
                setColumn(name = "unit", col = 3, type = "Text", error = "notUnit"),
                setColumn(name = "S1.1", col = 4, type = "Numeric"),
                setColumn(name = "S1.2", col = 5, type = "Numeric"),
                setColumn(name = "S1.3", col = 6, type = "Numeric"),
                setColumn(name = "S4.1", col = 7, type = "Numeric"),
                setColumn(name = "S4.2", col = 8, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "scenarioSST",
              description = "scenario table",
              sheet = "Scenarios",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "label", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "name", col = 3, type = "Text"),
                setColumn(name = "effect", col = 7, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE),
                setColumn(name = "probability", col = 8, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, error = c("Negative", "LargerOne"))
              )
    )
    
    ,setTable(keyword = "scenarioHealth",
              description = "health scenarios",
              sheet = "Health",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "label", col = 7, type = "Text", characteristic = "Primary key"),
                setColumn(name = "effect", col = 8, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE),
                setColumn(name = "probability", col = 9, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, error = c("Negative", "LargerOne"))
              )
    )
    
    ,setTable(keyword = "scenarioNL",
              description = "non-life scenarios",
              sheet = "Non Life",
              row = 18,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "label", col = 30, type = "Text", characteristic = "Primary key"),
                setColumn(name = "effect", col = 31, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE),
                setColumn(name = "probability", col = 32, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, error = c("Negative", "LargerOne"))
              )
    )
    
    ,setTable(keyword = "creditRiskExposure",
              description = "credit risk table",
              sheet = "Credit Risk Basel",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              isDropEmptyRows = TRUE,
              columns = list(
                setColumn(name = "rating", col = 7, type = "Text"),
                setColumn(name = "weight", col = 8, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE),
                setColumn(name = "weightedExposure", col = 12, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "creditRiskExposureReinsurance",
              description = "credit risk table",
              sheet = "Credit Risk Basel",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              isDropEmptyRows = TRUE,
              columns = list(
                setColumn(name = "class", col = 2, type = "Text", isValueRequired = FALSE, filterValue = "^A.7a$", characteristic = "AutoComplete"),
                setColumn(name = "rating", col = 7, type = "Text"),
                setColumn(name = "weight", col = 8, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE),
                setColumn(name = "weightedExposure", col = 12, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "creditRiskExposureHypo",
              description = "credit risk table",
              sheet = "Credit Risk Basel",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              isDropEmptyRows = TRUE,
              columns = list(
                setColumn(name = "class", col = 2, type = "Text", isValueRequired = FALSE, filterValue = "(^A.8$)|(^B.3$)", characteristic = "AutoComplete"),
                setColumn(name = "rating", col = 7, type = "Text"),
                setColumn(name = "weight", col = 8, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE),
                setColumn(name = "weightedExposure", col = 12, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "sensitivityHealth",
              description = "health insurance sensitivities table",
              sheet = "Health",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "name", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "sensitivity", col = 5, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, characteristic = "Named vector", error = "Negative")
              )
    )
    
    ,setTable(keyword = "exposureLife",
              description = "life insurance sensitivities table",
              sheet = "Life",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "name", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "sensitivity", col = 5, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, characteristic = "Named vector")
              )
    )
    
    # Only to load the sheet
    ,setTable(keyword = "dummy1",
              description = "(cell from the balance sheet)",
              sheet = "SST Balance Sheet",
              row = 1,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 16, type = "Text", isValueRequired = FALSE)
              )
    )
    
    ,setTable(keyword = "dummy2",
              description = "(cell from other data)",
              sheet = "Other Data",
              row = 1,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 5, type = "Text", isValueRequired = FALSE)
              )
    )
    
    ,setTable(keyword = "configurationVersionId",
              description = "version of the configuration file",
              sheet = "Configuration",
              row = 2,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 26, type = "Numeric")
              )
    )
    
    ,setTable(keyword = "participationValue",
              description = "value for participation",
              sheet = "Asset Prices",
              row = 7,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 6, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, error = "Negative")
              )
    )
    
    ,setTable(keyword = "MVM_Health",
              description = "MVM (before the nhmr factor)",
              sheet = "General Inputs",
              row = 15,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, error = "Negative")
              )
    )
    ,setTable(keyword = "MVM_NonLife",
              description = "MVM (before the nhmr factor)",
              sheet = "General Inputs",
              row = 14,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, error = "Negative")
              )
    )
    
    ,setTable(keyword = "PYrisk",
              description = "PY for non-captive NL companies",
              sheet = "Other Data",
              row = 15,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 5, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE)
              )
    )
    
    ,setTable(keyword = "CYrisk",
              description = "CY for non-captive NL companies",
              sheet = "Other Data",
              row = 19,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 5, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE)
              )
    )
    
    ,setTable(keyword = "selectedBranch",
              description = "branch",
              sheet = "Intro",
              row = 11,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 5, type = "Text")
              )
    )
    ,setTable(keyword = "fileNameSuffix",
              description = "suffix for the output files",
              sheet = "Intro",
              row = 21,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 5, type = "Text", isValueRequired = FALSE)
              )
    )
    
    ,setTable(keyword = "nonlifeMu",
              description = "lognormal parameter (mu) for nonlife",
              sheet = "Non Life",
              row = 18,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 7, type = "Numeric", isValueRequired = FALSE)
              )
    )
    
    ,setTable(keyword = "nonlifeSigma",
              description = "lognormal parameter (sigma) for nonlife",
              sheet = "Non Life",
              row = 18,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 8, type = "Numeric", isValueRequired = FALSE, error = "Negative")
              )
    )
    
    ,setTable(keyword = "nonlifeType",
              description = "type of nonlife input",
              sheet = "Non Life",
              row = 5,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 5, type = "Text", error = "notNonLifeType")
              )
    )
    
    ,setTable(keyword = "companyName",
              description = "name of the insurance company",
              sheet = "Intro",
              row = 9,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 5, type = "Text")
              )
    )
    
    ,setTable(keyword = "referenceCurrency",
              description = "SST currency",
              sheet = "General Inputs",
              row = 7,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Text", error = "notCurrency")
              )
    )
    
    ,setTable(keyword = "expectedInsuranceResult",
              description = "expected insurance result",
              sheet = "General Inputs",
              row = 9,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE)
              )
    )
    
    ,setTable(keyword = "daughterValue",
              description = "value of the dauther company seen as participation by the mother",
              sheet = "General Inputs",
              row = 20,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", isValueRequired = FALSE)
              )
    )
    
    
    ,setTable(keyword = "daughterScaling",
              description = "participation scaling factor (for the look-through approach)",
              sheet = "General Inputs",
              row = 21,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", isValueRequired = FALSE)
              )
    )
    
    ,setTable(keyword = "daughterApplyLLPO",
              description = "indicator for the application of the LLPO (for the look-through approach)",
              sheet = "General Inputs",
              row = 22,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Logical", isValueRequired = FALSE)
              )
    )
    
    ,setTable(keyword = "riskAsParticipation",
              description = "indicator for aggregating daughter market risk under the 'participation' risk factor",
              sheet = "General Inputs",
              row = 23,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Logical", isValueRequired = FALSE)
              )
    )
    
    ,setTable(keyword = "RTK_goingConcern",
              description = "going concern RBC",
              sheet = "RBC",
              row = 37,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 5, type = "Numeric", warning = c("Negative", "Zero"))
              )
    )
    ,setTable(keyword = "RTK_runOff",
              description = "run off RBC",
              sheet = "RBC",
              row = 46,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 5, type = "Numeric", warning = c("Negative", "Zero"))
              )
    )
    ,setTable(keyword = "additionalEffectsOnTC",
              description = "additional effects on the target capital",
              sheet = "General Inputs",
              row = 8,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", isValueRequired = FALSE, isReplacementZero = TRUE, warning = "Negative")
              )
    )
    
    ,setTable(keyword = "expectedFinancialResultFactor",
              description = "additional effects on the target capital",
              sheet = "General Parameters",
              row = 6,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", error = "Negative")
              )
    )
    ,setTable(keyword = "param_creditRiskFactor",
              description = "credit risk",
              sheet = "General Parameters",
              row = 7,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", error = "Negative")
              )
    )
    ,setTable(keyword = "param_costOfCapital",
              description = "cost of capital",
              sheet = "General Parameters",
              row = 8,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", error = "Negative")
              )
    )
    ,setTable(keyword = "param_participationVolatility",
              description = "volatility of immaterial participations",
              sheet = "General Parameters",
              row = 10,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", error = "Negative")
              )
    )
    ,setTable(keyword = "param_nhmrFactor",
              description = "additional effects on the target capital",
              sheet = "General Parameters",
              row = 9,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Numeric", error = "Negative")
              )
    )
    ,setTable(keyword = "migrationMatrix",
              description = "migration matrix",
              sheet = "Credit Risk Parameters",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              
              columns = list(
                setColumn(name = "category", col = 5, type = "Integer", characteristic = "Primary key"),
                setColumn(name = paste0(1:9), col = 6, type = "Numeric", error = "Negative")
              )
    )
    ,setTable(keyword = "spreadCurve",
              description = "cumulating spread curve",
              sheet = "Credit Risk Parameters",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "category", col = 16, type = "Integer", characteristic = "Primary key"),
                setColumn(name = "curve", col = 17, type = "Numeric", error = "notIncreasing", warning = "Negative")
              )
    )
    ,setTable(keyword = "mappingLGD",
              description = "mapping LGD",
              sheet = "Credit Risk Parameters",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = FALSE,
              columns = list(
                setColumn(name = "class", col = 2, type = "Text", characteristic = "Primary key"),
                setColumn(name = "LGD", col = 3, type = "Numeric", error = c("Negative", "LargerOne"))
              )
    )
    # ,setTable(keyword = "mappingCategory",
    #           description = "mapping for Basel categories",
    #           sheet = "Parameters",
    #           row = 20,
    #           isValue = FALSE,
    #           isEmptyAllowed = FALSE,
    #           isCorrelation = FALSE,
    #           columns = list(
    #             setColumn(name = "class", col = 5, type = "Text", characteristic = "Primary key"),
    #             setColumn(name = paste0(1:8), col = 6, type = "Numeric", error = "Negative", characteristic = "Transpose")
    #           )
    # )
    
    ,setTable(keyword = "portfolioCreditRisk",
              description = "portfolio without coupons",
              sheet = "Credit Risk Merton",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "positionId", col = 2, type = "Text"),
                setColumn(name = "positionName", col = 3, type = "Text"),
                setColumn(name = "includeInCreditRisk", col = 4, type = "Logical", filterValue = "^Yes$"), #Filter is applied before conversion to logical
                setColumn(name = "counterpartyId", col = 5, type = "Text"),
                setColumn(name = "counterpartyName", col = 6, type = "Text"),
                setColumn(name = "rating", col = 7, type = "Integer", error = "notRating"),
                setColumn(name = "basel", col = 9, type = "Text", error = "notLGDClass"),
                setColumn(name = "migration", col = 10, type = "Logical"),
                setColumn(name = "currency", col = 11, type = "Text", error = "notCurrency"),
                setColumn(name = "scalingCF", col = 12, type = "Numeric", error =  c("Negative", "Zero", "LargerOne"), isValueRequired = FALSE),
                setColumn(name = "scalingLGD", col = 13, type = "Numeric", error = c("Negative", "Zero", "LargerOne"), isValueRequired = FALSE),
                setColumn(name = "marketValue", col = 14, type = "Numeric", error = "Negative"),
                setColumn(name = paste0("Y", 1:50), col = 15, type = "Numeric", isValueRequired = FALSE, error = "Negative")
              )
    )
    ,setTable(keyword = "portfolioCreditRiskCheck",
              description = "portfolio without coupons",
              sheet = "Credit Risk Merton",
              row = 7,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "positionId", col = 2, type = "Text"),
                setColumn(name = "positionName", col = 3, type = "Text"),
                setColumn(name = "includeInCreditRisk", col = 4, type = "Logical"),
                setColumn(name = "counterpartyId", col = 5, type = "Text"),
                setColumn(name = "counterpartyName", col = 6, type = "Text"),
                setColumn(name = "rating", col = 7, type = "Integer", error = "notRating"),
                setColumn(name = "basel", col = 9, type = "Text"),
                setColumn(name = "migration", col = 10, type = "Logical"),
                setColumn(name = "currency", col = 11, type = "Text", error = "notCurrency"),
                setColumn(name = "marketValue", col = 14, type = "Numeric", error = "Negative"),
                setColumn(name = paste0("Y", 1:50), col = 15, type = "Numeric", isValueRequired = FALSE, error = "Negative")
              )
    )
    
    ,setTable(keyword = "captiveCY_deterministic",
              description = "captive current year parameters",
              sheet = "Non Life",
              row = 18,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "branch", col = 10, type = "Text", characteristic = "Primary key"),
                setColumn(name = "branchType", col = 11, type = "Text", filterValue = "^Maximum possible loss$"),
                setColumn(name = "maximumPossibleLoss", col = 12, type = "Numeric", warning = c("Negative", "Zero")),
                setColumn(name = "expectedLoss", col = 13, type = "Numeric", warning = c("Negative", "Zero")),
                setColumn(name = "normalLossNumber", col = 14, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "normalLossSize", col = 15, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "normalLossSigma", col = 16, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "largeLossNumber", col = 17, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "largeLossThreshold", col = 18, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "largeLossShape", col = 19, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "AAL", col = 20, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "EEL", col = 21, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "AAD", col = 22, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "EED", col = 23, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "QS", col = 24, type = "Numeric", isValueRequired = FALSE, error = "notEmpty")
              )
    )
    
    ,setTable(keyword = "captiveCY_stochastic",
              description = "captive current year parameters",
              sheet = "Non Life",
              row = 18,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "branch", col = 10, type = "Text", characteristic = "Primary key"),
                setColumn(name = "branchType", col = 11, type = "Text", filterValue = "^Ground-up loss$"),
                setColumn(name = "maximumPossibleLoss", col = 12, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "expectedLoss", col = 13, type = "Numeric", isValueRequired = FALSE, error = "notEmpty"),
                setColumn(name = "normalLossNumber", col = 14, type = "Numeric", error = c("Negative", "Zero", "notSmallValue"), isValueRequired = FALSE),
                setColumn(name = "normalLossSize", col = 15, type = "Numeric", error = c("Negative", "Zero"), isValueRequired = FALSE),
                setColumn(name = "normalLossSigma", col = 16, type = "Numeric", error = c("Negative", "Zero"), isValueRequired = FALSE),
                setColumn(name = "largeLossNumber", col = 17, type = "Numeric", error = c("Negative", "Zero", "notSmallValue"), isValueRequired = FALSE),
                setColumn(name = "largeLossThreshold", col = 18, type = "Numeric", error = c("Negative", "Zero"), isValueRequired = FALSE),
                setColumn(name = "largeLossShape", col = 19, type = "Numeric", error = c("Negative", "Zero"), isValueRequired = FALSE),
                setColumn(name = "AAL", col = 20, type = "Numeric", error = c("Negative", "Zero"), isValueRequired = FALSE),
                setColumn(name = "EEL", col = 21, type = "Numeric", error = c("Negative", "Zero"), isValueRequired = FALSE),
                setColumn(name = "AAD", col = 22, type = "Numeric", error = c("Negative"), isReplacementZero = TRUE, isValueRequired = FALSE),
                setColumn(name = "EED", col = 23, type = "Numeric", error = c("Negative"), isReplacementZero = TRUE, isValueRequired = FALSE),
                setColumn(name = "QS", col = 24, type = "Numeric", error = c("Negative", "Zero", "LargerOne"), isValueRequired = FALSE)
              )
    )
    
    ,setTable(keyword = "captiveCY",
              description = "captive current year parameters",
              sheet = "Non Life",
              row = 18,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "branch", col = 10, type = "Text", characteristic = "Primary key"),
                setColumn(name = "branchType", col = 11, type = "Text", error = "notCaptiveBranchType"),
                setColumn(name = "maximumPossibleLoss", col = 12, type = "Numeric", isValueRequired = FALSE),
                setColumn(name = "expectedLoss", col = 13, type = "Numeric", isValueRequired = FALSE),
                setColumn(name = "normalLossNumber", col = 14, type = "Numeric", isValueRequired = FALSE),
                setColumn(name = "normalLossSize", col = 15, type = "Numeric", isValueRequired = FALSE),
                setColumn(name = "normalLossSigma", col = 16, type = "Numeric", isValueRequired = FALSE),
                setColumn(name = "largeLossNumber", col = 17, type = "Numeric", isValueRequired = FALSE),
                setColumn(name = "largeLossThreshold", col = 18, type = "Numeric", isValueRequired = FALSE),
                setColumn(name = "largeLossShape", col = 19, type = "Numeric", isValueRequired = FALSE),
                setColumn(name = "AAL", col = 20, type = "Numeric", error = c("Negative", "Zero"), isValueRequired = FALSE),
                setColumn(name = "EEL", col = 21, type = "Numeric", error = c("Negative", "Zero"), isValueRequired = FALSE),
                setColumn(name = "AAD", col = 22, type = "Numeric", error = c("Negative"), isReplacementZero = TRUE, isValueRequired = FALSE),
                setColumn(name = "EED", col = 23, type = "Numeric", error = c("Negative"), isReplacementZero = TRUE, isValueRequired = FALSE),
                setColumn(name = "QS", col = 24, type = "Numeric", error = c("Negative", "Zero"), isValueRequired = FALSE)
              )
    )  
    
    ,setTable(keyword = "captivePY",
              description = "captive previous year parameters",
              sheet = "Non Life",
              row = 18,
              isValue = FALSE,
              isEmptyAllowed = TRUE,
              columns = list(
                setColumn(name = "branch", col = 26, type = "Text", characteristic = "Primary key"),
                setColumn(name = "reserve", col = 27, type = "Numeric", error = "Negative", warning = "Zero"),
                setColumn(name = "cov", col = 28, type = "Numeric", error = c("Negative", "Zero"))
              )
    )
    
    ,setTable(keyword = "useNewCreditRiskModel",
              description = "usage of new (field-test) credit risk model",
              sheet = "General Inputs",
              row = 24,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 4, type = "Logical")
              )
    )
    ,setTable(keyword = "rho",
              description = "correlation between idiosyncratic and systematic risks",
              sheet = "Credit Risk Parameters",
              row = 7,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 21, type = "Numeric", error = c("Negative", "LargerOne"))
              )
    )
    ,setTable(keyword = "corr",
              description = "correlation between Basel3 Term and Stochastic Bond model",
              sheet = "Credit Risk Parameters",
              row = 8,
              isValue = TRUE,
              columns = list(
                setColumn(name = "none", col = 21, type = "Numeric", error = c("Negative", "LargerOne"))
              )
    )
  )
  
  config <- rbindlist(TemplateConfig)
  fwrite(config, "inst/extdata/wb.csv")
}